import { Directive, Self, Inject, Input, forwardRef, ElementRef, Injector, HostListener, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR, CheckboxControlValueAccessor, DefaultValueAccessor, SelectControlValueAccessor, SelectMultipleControlValueAccessor } from '@angular/forms';
import { noop, isNil } from '@s-libs/micro-dash';
import { provideValueAccessor } from '@s-libs/ng-core';

class NasModelDirective {
    constructor(valueAccessors) {
        this.valueAccessor = valueAccessors[0];
    }
    set nasModel(store) {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
        this.store = store;
        this.subscription = store.$.subscribe((value) => {
            this.valueAccessor.writeValue(value);
        });
    }
    set disabled(isDisabled) {
        var _a, _b;
        (_b = (_a = this.valueAccessor).setDisabledState) === null || _b === void 0 ? void 0 : _b.call(_a, isDisabled);
    }
    ngAfterViewInit() {
        this.valueAccessor.registerOnChange((value) => {
            this.store.set(value);
        });
    }
    ngOnDestroy() {
        this.subscription.unsubscribe();
    }
}
NasModelDirective.decorators = [
    { type: Directive, args: [{ selector: '[nasModel]' },] }
];
NasModelDirective.ctorParameters = () => [
    { type: Array, decorators: [{ type: Self }, { type: Inject, args: [NG_VALUE_ACCESSOR,] }] }
];
NasModelDirective.propDecorators = {
    nasModel: [{ type: Input }],
    disabled: [{ type: Input }]
};

/** @hidden */
class CheckboxValueAccessorDirective extends CheckboxControlValueAccessor {
}
CheckboxValueAccessorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[type=checkbox][nasModel]',
                // tslint:disable-next-line:no-host-metadata-property
                host: {
                    '(change)': 'onChange($event.target.checked)',
                    '(blur)': 'onTouched()',
                },
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => CheckboxValueAccessorDirective),
                        multi: true,
                    },
                ],
            },] }
];

/** @hidden */
class InputValueAccessorDirective extends DefaultValueAccessor {
}
InputValueAccessorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input:not([type=checkbox]):not([type=number]):not([type=radio]):not([type=range])[nasModel],textarea[nasModel]',
                // tslint:disable-next-line:no-host-metadata-property
                host: {
                    '(input)': '$any(this)._handleInput($event.target.value)',
                    '(blur)': 'onTouched()',
                    '(compositionstart)': '$any(this)._compositionStart()',
                    '(compositionend)': '$any(this)._compositionEnd($event.target.value)',
                },
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => InputValueAccessorDirective),
                        multi: true,
                    },
                ],
            },] }
];

/** @hidden */
class AbstractValueAccessorDirective {
    constructor(injector) {
        this.onTouchedFn = noop;
        this.elementRef = injector.get(ElementRef);
    }
    registerOnChange(fn) {
        this.onChangeFn = fn;
    }
    registerOnTouched(fn) {
        this.onTouchedFn = fn;
    }
    get element() {
        return this.elementRef.nativeElement;
    }
}
AbstractValueAccessorDirective.decorators = [
    { type: Directive }
];
AbstractValueAccessorDirective.ctorParameters = () => [
    { type: Injector }
];

/** @hidden */
class AbstractInputValueAccessorDirective extends AbstractValueAccessorDirective {
    onChange() {
        this.onChangeFn(this.element.value);
    }
    onBlur() {
        this.onTouchedFn();
    }
    writeValue(value) {
        this.element.value = value;
    }
    setDisabledState(isDisabled) {
        this.element.disabled = isDisabled;
    }
}
AbstractInputValueAccessorDirective.decorators = [
    { type: Directive }
];
AbstractInputValueAccessorDirective.propDecorators = {
    onChange: [{ type: HostListener, args: ['change',] }, { type: HostListener, args: ['input',] }],
    onBlur: [{ type: HostListener, args: ['blur',] }]
};

/** @hidden */
class NumberValueAccessorDirective extends AbstractInputValueAccessorDirective {
    registerOnChange(fn) {
        this.onChangeFn = (value) => {
            fn(value === '' ? null : parseFloat(value));
        };
    }
    writeValue(value) {
        // The value needs to be normalized for IE9, otherwise it is set to 'null' when null
        this.element.value = isNil(value) ? '' : value.toString();
    }
}
NumberValueAccessorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[type=number][nasModel]',
                providers: [provideValueAccessor(NumberValueAccessorDirective)],
            },] }
];

/** @hidden */
class RadioValueAccessorDirective extends AbstractInputValueAccessorDirective {
    writeValue(obj) {
        // delay because `button.value` may not be set yet as the component is being initialized
        Promise.resolve().then(() => {
            this.element.checked = this.element.value === obj;
        });
    }
}
RadioValueAccessorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[type=radio][nasModel]',
                providers: [provideValueAccessor(RadioValueAccessorDirective)],
            },] }
];

/** @hidden */
class RangeValueAccessorDirective extends AbstractInputValueAccessorDirective {
    registerOnChange(fn) {
        this.onChangeFn = (value) => {
            fn(parseFloat(value));
        };
    }
}
RangeValueAccessorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[type=range][nasModel]',
                providers: [provideValueAccessor(RangeValueAccessorDirective)],
            },] }
];

/** @hidden */
class SelectValueAccessorDirective extends SelectControlValueAccessor {
}
SelectValueAccessorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'select:not([multiple])[nasModel]',
                // tslint:disable-next-line:no-host-metadata-property
                host: {
                    '(change)': 'onChange($event.target.value)',
                    '(blur)': 'onTouched()',
                },
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => SelectValueAccessorDirective),
                        multi: true,
                    },
                    {
                        provide: SelectControlValueAccessor,
                        useExisting: forwardRef(() => SelectValueAccessorDirective),
                    },
                ],
            },] }
];

/** @hidden */
class SelectMultipleValueAccessorDirective extends SelectMultipleControlValueAccessor {
}
SelectMultipleValueAccessorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'select[multiple][nasModel]',
                // tslint:disable-next-line:no-host-metadata-property
                host: { '(change)': 'onChange($event.target)', '(blur)': 'onTouched()' },
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => SelectMultipleValueAccessorDirective),
                        multi: true,
                    },
                    {
                        provide: SelectMultipleControlValueAccessor,
                        useExisting: forwardRef(() => SelectMultipleValueAccessorDirective),
                    },
                ],
            },] }
];

/** @hidden */
const exportedDirectives = [
    CheckboxValueAccessorDirective,
    InputValueAccessorDirective,
    NasModelDirective,
    NumberValueAccessorDirective,
    RadioValueAccessorDirective,
    RangeValueAccessorDirective,
    SelectValueAccessorDirective,
    SelectMultipleValueAccessorDirective,
];
class NasModelModule {
}
NasModelModule.decorators = [
    { type: NgModule, args: [{ declarations: exportedDirectives, exports: exportedDirectives },] }
];

/*
 * Public API Surface of ng-app-state
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NasModelModule, CheckboxValueAccessorDirective as ɵa, InputValueAccessorDirective as ɵb, NasModelDirective as ɵc, NumberValueAccessorDirective as ɵd, AbstractInputValueAccessorDirective as ɵe, AbstractValueAccessorDirective as ɵf, RadioValueAccessorDirective as ɵg, RangeValueAccessorDirective as ɵh, SelectValueAccessorDirective as ɵi, SelectMultipleValueAccessorDirective as ɵj };
//# sourceMappingURL=s-libs-ng-app-state.js.map
